@@ -4,15 +4,15 @@ use namespace::autoclean;
 
 =head1 NAME
 
-URL::List - Object-oriented methods of handling list of URLs.
+URL::List - Helper class for creating distributed lists of URLs based on their host name, domain name or TLDs.
 
 =head1 VERSION
 
-Version 0.12
+Version 0.13
 
 =cut
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 use Carp;
 use Domain::PublicSuffix;
@@ -297,6 +297,141 @@ sub distributed_by_tld {
     return $self->distributions->{tld};
 }
 
+sub _blocks_by {
+    my $self = shift;
+    my $dist = shift;
+
+    my @blocks = ();
+
+    while ( keys %{$dist} ) {
+        my @urls = ();
+
+        foreach my $key ( sort keys %{$dist} ) {
+            if ( my $url = shift @{$dist->{$key}} ) {
+                push( @urls, $url );
+            }
+            else {
+                delete $dist->{$key};
+            }
+        }
+
+        if ( @urls ) {
+            push( @blocks, \@urls );
+        }
+    }
+
+    return \@blocks;
+}
+
+=head2 blocks_by_host, blocks_by_domain, blocks_by_tld
+
+Returns "blocks" of URLs distributed by their host/domain/TLD, i.e. an array
+reference of array references containing URLs distributed as evenly as possible;
+
+    my $list = URL::List->new(
+        urls => [qw(
+            http://www.businessinsider.com/1.html
+            http://www.businessinsider.com/2.html
+            http://www.businessinsider.com/3.html
+            http://www.engadget.com/1.html
+            http://www.engadget.com/2.html
+            http://www.engadget.com/3.html
+            http://www.engadget.com/4.html
+            http://www.independent.co.uk/1.html
+            http://www.independent.co.uk/2.html
+            http://www.pcmag.com/1.html
+            http://www.pcmag.com/2.html
+            http://www.pcmag.com/3.html
+            http://www.technologyreview.com/1.html
+            http://www.technologyreview.com/2.html
+            http://www.technologyreview.com/3.html
+            http://www.technologyreview.com/4.html
+            http://www.zdnet.com/1.html
+            http://www.zdnet.com/2.html
+            http://www.zdnet.com/3.html
+        )],
+    );
+
+    # $list->blocks_by_host = [
+    #     [qw(
+    #         http://www.businessinsider.com/1.html
+    #         http://www.engadget.com/1.html
+    #         http://www.independent.co.uk/1.html
+    #         http://www.pcmag.com/1.html
+    #         http://www.technologyreview.com/1.html
+    #         http://www.zdnet.com/1.html
+    #     )],
+    #
+    #     [qw(
+    #         http://www.businessinsider.com/2.html
+    #         http://www.engadget.com/2.html
+    #         http://www.independent.co.uk/2.html
+    #         http://www.pcmag.com/2.html
+    #         http://www.technologyreview.com/2.html
+    #         http://www.zdnet.com/2.html
+    #     )],
+    #
+    #     [qw(
+    #         http://www.businessinsider.com/3.html
+    #         http://www.engadget.com/3.html
+    #         http://www.pcmag.com/3.html
+    #         http://www.technologyreview.com/3.html
+    #         http://www.zdnet.com/3.html
+    #     )],
+    #
+    #     [qw(
+    #         http://www.engadget.com/4.html
+    #         http://www.technologyreview.com/4.html
+    #     )],
+    # ],
+
+This is useful if you want to crawl many URLs, but also want to pause between
+each visit to host/domain/TLD;
+
+    my $list = URL::List->new( urls => [...] );
+
+    foreach my $urls ( @{$list->blocks_by_domain} ) {
+        # get $urls in parallel, you will only visit each domain once, or you
+        # can delegate $urls to other workers (crawlers) to spread load etc.
+
+        sleep( 5 ); # let's be nice and pause
+    }
+
+=cut
+
+has 'blocks_by_host' => (
+    isa => 'ArrayRef[ArrayRef]',
+    is => 'ro',
+    lazy => 1,
+    default => sub {
+        my $self = shift;
+
+        return $self->_blocks_by( $self->distributed_by_host );
+    },
+);
+
+has 'blocks_by_domain' => (
+    isa => 'ArrayRef[ArrayRef]',
+    is => 'ro',
+    lazy => 1,
+    default => sub {
+        my $self = shift;
+
+        return $self->_blocks_by( $self->distributed_by_domain );
+    },
+);
+
+has 'blocks_by_tld' => (
+    isa => 'ArrayRef[ArrayRef]',
+    is => 'ro',
+    lazy => 1,
+    default => sub {
+        my $self = shift;
+
+        return $self->_blocks_by( $self->distributed_by_tld );
+    },
+);
+
 #
 # The End
 #
